@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,10 +19,10 @@ from .core import DataFrameDeviceConversionBase
 class DataFrameToGPU(DataFrameDeviceConversionBase):
     _op_type_ = OperandDef.TO_GPU
 
-    def __init__(self, dtypes=None, gpu=None, sparse=None, output_types=None, **kw):
-        super().__init__(_dtypes=dtypes, _gpu=gpu, _sparse=sparse, _output_types=output_types, **kw)
-        if not self._gpu:
-            self._gpu = True
+    def __init__(self, dtypes=None, output_types=None, **kw):
+        super().__init__(_dtypes=dtypes, _output_types=output_types, **kw)
+        if not self.gpu:
+            self.gpu = True
 
     @classmethod
     def execute(cls, ctx, op):
