@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ from pandas._libs.tslibs import timezones
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core import OutputType
-from ...serialize import AnyField, Int64Field, BoolField, StringField
+from ...serialization.serializables import AnyField, Int64Field, BoolField, StringField
 from ...tensor.utils import decide_chunk_sizes
 from ..operands import DataFrameOperand, DataFrameOperandMixin
 from ..utils import parse_index
