@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,13 +28,13 @@ from ... import opcodes as OperandDef
 from ...config import options
 from ...core import OutputType
 from ...lib.filesystem import get_fs, open_file, file_size, glob
-from ...serialize import StringField, DictField, ListField, Int32Field, Int64Field, BoolField, AnyField
+from ...serialization.serializables import StringField, DictField, ListField, Int32Field, \
+    Int64Field, BoolField, AnyField
 from ...utils import parse_readable_size, lazy_import, FixedSizeFileObject
 from ..arrays import ArrowStringDtype
-from ..core import IndexValue
-from ..utils import parse_index, build_empty_df, standardize_range_index, \
-    to_arrow_dtypes, contain_arrow_dtype
-from .core import HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin
+from ..utils import parse_index, build_empty_df, to_arrow_dtypes, contain_arrow_dtype
+from .core import IncrementalIndexDatasource, ColumnPruneSupportedDataSourceMixin, \
+    IncrementalIndexDataSourceMixin
 
 
 cudf = lazy_import('cudf', globals=globals())
@@ -81,7 +81,9 @@ def _find_chunk_start_end(f, offset, size):
     return start, end
 
 
-class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin):
+class DataFrameReadCSV(IncrementalIndexDatasource,
+                       ColumnPruneSupportedDataSourceMixin,
+                       IncrementalIndexDataSourceMixin):
     _op_type_ = OperandDef.READ_CSV
 
     _path = AnyField('path')
@@ -90,7 +92,7 @@ class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMi
     _header = AnyField('header')
     _index_col = Int32Field('index_col')
     _compression = StringField('compression')
-    _usecols = ListField('usecols')
+    _usecols = AnyField('usecols')
     _offset = Int64Field('offset')
     _size = Int64Field('size')
     _nrows = Int64Field('nrows')
@@ -101,18 +103,16 @@ class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMi
 
     def __init__(self, path=None, names=None, sep=None, header=None, index_col=None,
                  compression=None, usecols=None, offset=None, size=None, nrows=None,
-                 gpu=None, keep_usecols_order=None, incremental_index=None,
-                 use_arrow_dtype=None, storage_options=None,
-                 memory_scale=None, **kw):
+                 keep_usecols_order=None, incremental_index=None,
+                 use_arrow_dtype=None, storage_options=None, **kw):
         super().__init__(_path=path, _names=names, _sep=sep, _header=header,
                          _index_col=index_col, _compression=compression,
                          _usecols=usecols, _offset=offset, _size=size, _nrows=nrows,
-                         _gpu=gpu, _incremental_index=incremental_index,
+                         _incremental_index=incremental_index,
                          _keep_usecols_order=keep_usecols_order,
                          _use_arrow_dtype=use_arrow_dtype,
                          _storage_options=storage_options,
-                         _output_types=[OutputType.dataframe],
-                         _memory_scale=memory_scale, **kw)
+                         _output_types=[OutputType.dataframe], **kw)
 
     @property
     def path(self):
@@ -237,9 +237,6 @@ class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMi
                 index_num += 1
                 offset += chunk_bytes
 
-        if op.incremental_index and len(out_chunks) > 1 and \
-                isinstance(df.index_value._index_value, IndexValue.RangeIndex):
-            out_chunks = standardize_range_index(out_chunks)
         new_op = op.copy()
         nsplits = ((np.nan,) * len(out_chunks), (df.shape[1],))
         return new_op.new_dataframes(None, df.shape, dtypes=dtypes,
@@ -335,7 +332,6 @@ class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMi
                     df = df[op.usecols]
             else:
                 df = cls._cudf_read_csv(op) if op.gpu else cls._pandas_read_csv(f, op)
-
         ctx[out_df.key] = df
 
     def estimate_size(cls, ctx, op):
@@ -350,7 +346,7 @@ class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMi
 
 def read_csv(path, names=None, sep=',', index_col=None, compression=None, header='infer',
              dtype=None, usecols=None, nrows=None, chunk_bytes='64M', gpu=None, head_bytes='100k',
-             head_lines=None, incremental_index=False, use_arrow_dtype=None,
+             head_lines=None, incremental_index=True, use_arrow_dtype=None,
              storage_options=None, memory_scale=None, **kwargs):
     r"""
     Read a comma-separated values (csv) file into DataFrame.
@@ -363,7 +359,7 @@ def read_csv(path, names=None, sep=',', index_col=None, compression=None, header
         Any valid string path is acceptable. The string could be a URL. Valid
         URL schemes include http, ftp, s3, and file. For file URLs, a host is
         expected. A local file could be: file://localhost/path/to/table.csv,
-        you can alos read from external resources using a URL like:
+        you can also read from external resources using a URL like:
         hdfs://localhost:8020/test.csv.
         If you want to pass in a path object, pandas accepts any ``os.PathLike``.
         By file-like object, we refer to objects with a ``read()`` method, such as
@@ -494,7 +490,7 @@ def read_csv(path, names=None, sep=',', index_col=None, compression=None, header
         * dict, e.g. {'foo' : [1, 3]} -> parse columns 1, 3 as date and call
           result 'foo'
         If a column or index cannot be represented as an array of datetimes,
-        say because of an unparseable value or a mixture of timezones, the column
+        say because of an unparsable value or a mixture of timezones, the column
         or index will be returned unaltered as an object data type. For
         non-standard datetime parsing, use ``pd.to_datetime`` after
         ``pd.read_csv``. To parse an index or column with a mixture of timezones,
@@ -610,8 +606,9 @@ def read_csv(path, names=None, sep=',', index_col=None, compression=None, header
         Number of bytes to use in the head of file, mainly for data inference.
     head_lines: int, optional
         Number of lines to use in the head of file, mainly for data inference.
-    incremental_index: bool, default False
-        Create a new RangeIndex if csv doesn't contain index columns.
+    incremental_index: bool, default True
+        If index_col not specified, ensure range index incremental,
+        gain a slightly better performance if setting False.
     use_arrow_dtype: bool, default None
         If True, use arrow dtype to store columns.
     storage_options: dict, optional
@@ -630,9 +627,13 @@ def read_csv(path, names=None, sep=',', index_col=None, compression=None, header
     Examples
     --------
     >>> import mars.dataframe as md
+    >>> from mars.lib.filesystem.oss import build_oss_path
     >>> md.read_csv('data.csv')  # doctest: +SKIP
     >>> # read from HDFS
     >>> md.read_csv('hdfs://localhost:8020/test.csv')  # doctest: +SKIP
+    >>> # read from OSS
+    >>> auth_path = build_oss_path(file_path, access_key_id, access_key_secret, end_point)
+    >>> md.read_csv(auth_path)
     """
     # infer dtypes and columns
     if isinstance(path, (list, tuple)):
