@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,12 +20,11 @@ import numpy as np
 
 from ... import opcodes as OperandDef
 from ...config import options
-from ...core import Base, Entity, OutputType
-from ...serialize import AnyField, Int32Field, BoolField
+from ...core import ENTITY_TYPE, OutputType, recursive_tile
+from ...serialization.serializables import AnyField, Int32Field, BoolField
 from ...tensor.core import TENSOR_TYPE, TENSOR_CHUNK_TYPE
 from ...tensor.datasource import tensor as astensor
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...utils import has_unknown_shape
 from ..align import align_dataframe_series, align_dataframe_dataframe
 from ..core import SERIES_TYPE, SERIES_CHUNK_TYPE, DATAFRAME_TYPE, DATAFRAME_CHUNK_TYPE
 from ..merge import DataFrameConcat
@@ -234,9 +233,9 @@ class DataFrameIndex(DataFrameOperand, DataFrameOperandMixin):
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
-        if isinstance(self._col_names, (Base, Entity)):
+        if isinstance(self._col_names, ENTITY_TYPE):
             self._col_names = self._inputs[0]
-        if isinstance(self._mask, (Base, Entity)):
+        if isinstance(self._mask, ENTITY_TYPE):
             self._mask = self._inputs[-1]
 
     def __call__(self, df):
@@ -270,7 +269,7 @@ class DataFrameIndex(DataFrameOperand, DataFrameOperandMixin):
         if op.col_names is not None:
             return cls.tile_with_columns(op)
         else:
-            return cls.tile_with_mask(op)
+            return (yield from cls.tile_with_mask(op))
 
     @classmethod
     def tile_with_mask(cls, op):
@@ -291,7 +290,8 @@ class DataFrameIndex(DataFrameOperand, DataFrameOperandMixin):
             else:
                 # tensor
                 nsplits = in_df.nsplits
-                mask = mask.rechunk(nsplits[:mask.ndim])._inplace_tile()
+                mask = yield from recursive_tile(
+                    mask.rechunk(nsplits[:mask.ndim]))
                 out_shape = in_df.chunk_shape
                 df_chunks = in_df.chunks
                 mask_chunks = mask.chunks
@@ -311,7 +311,8 @@ class DataFrameIndex(DataFrameOperand, DataFrameOperandMixin):
                                                             columns_value=df_chunk.columns_value)
                 out_chunks.append(out_chunk)
         else:
-            check_chunks_unknown_shape([in_df], TilesError)
+            if has_unknown_shape(in_df):
+                yield
             nsplits_acc = np.cumsum((0,) + in_df.nsplits[0])
             for idx in range(in_df.chunk_shape[0]):
                 for idxj in range(in_df.chunk_shape[1]):
@@ -401,6 +402,12 @@ class DataFrameIndex(DataFrameOperand, DataFrameOperandMixin):
                 mask = mask[df.columns.tolist()]
             ctx[op.outputs[0].key] = df[mask]
 
+    @classmethod
+    def estimate_size(cls, ctx: dict, op: "DataFrameIndex"):
+        super().estimate_size(ctx, op)
+        result_size = ctx[op.outputs[0].key][0]
+        ctx[op.outputs[0].key] = (result_size, result_size)
+
 
 _list_like_types = (list, np.ndarray, SERIES_TYPE, pd.Series, TENSOR_TYPE)
 
