@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,9 +20,8 @@ import numpy as np
 import pandas as pd
 from pandas.core.dtypes.cast import find_common_type
 
-from ...core import TileableEntity, Chunk, OutputType
-from ...operands import OperandStage
-from ...tiles import TilesError
+from ...core import Tileable, Chunk, OutputType, recursive_tile
+from ...core.operand import OperandStage
 from ...tensor.core import TENSOR_TYPE
 from ...tensor.indexing.index_lib import IndexHandlerContext, IndexHandler, \
     IndexInfo, IndexType, ChunkIndexInfo as ChunkIndexInfoBase, \
@@ -33,7 +32,7 @@ from ...tensor.indexing.index_lib import IndexHandlerContext, IndexHandler, \
 from ...tensor.utils import split_indexes_into_chunks, calc_pos, \
     filter_inputs, slice_split, calc_sliced_size, to_numpy, \
     normalize_chunk_sizes
-from ...utils import check_chunks_unknown_shape, classproperty, recursive_tile
+from ...utils import classproperty, has_unknown_shape
 from ..core import SERIES_CHUNK_TYPE, SERIES_TYPE, IndexValue
 from ..utils import parse_index
 from .utils import convert_labels_into_positions
@@ -92,7 +91,7 @@ class LabelFancyIndexInfo(IndexInfo):
 
 
 class DataFrameIndexHandlerContext(IndexHandlerContext):
-    def set_tileable(self, tileable: TileableEntity):
+    def set_tileable(self, tileable: Tileable):
         for chunk in tileable.chunks:
             self.chunk_index_to_info[chunk.index] = ChunkIndexInfo()
 
@@ -120,7 +119,7 @@ class DataFrameIndexHandlerContext(IndexHandlerContext):
                      chunk_index_info: ChunkIndexInfo) -> Chunk:
         chunk_op = self.op.copy().reset_key()
         chunk_op._indexes = indexes = chunk_index_info.indexes
-        chunk_op._stage = OperandStage.map
+        chunk_op.stage = OperandStage.map
 
         chunk_input = self.tileable.cix[chunk_index]
         chunk_inputs = filter_inputs([chunk_input] + indexes)
@@ -228,8 +227,7 @@ class LabelSliceIndexHandler(IndexHandler):
 
         if check:
             if any(np.isnan(ns) for ns in tileable.nsplits[input_axis]):
-                raise TilesError(f'Input tileable {tileable} has chunks with unknown shape '
-                                 f'on axis {input_axis}')
+                yield []
 
     def set_chunk_index_info(cls,
                              context: IndexHandlerContext,
@@ -267,7 +265,7 @@ class LabelSliceIndexHandler(IndexHandler):
         chunk_index_info.set(ChunkIndexAxisInfo(**kw))
 
     def _process_has_value_index(self,
-                                 tileable: TileableEntity,
+                                 tileable: Tileable,
                                  index_info: IndexInfo,
                                  index_value,
                                  input_axis: int,
@@ -402,8 +400,7 @@ class LabelIndexHandler(IndexHandler):
             index_value = [tileable.index_value, tileable.columns_value][input_axis]
         if index_value.has_value():
             if any(np.isnan(ns) for ns in tileable.nsplits[input_axis]):
-                raise TilesError(f'Input tileable {tileable} has chunks with unknown shape '
-                                 f'on axis {input_axis}')
+                yield []
 
     def process(self,
                 index_info: IndexInfo,
@@ -463,7 +460,7 @@ class DataFrameIndexHandler:
 
         dtypes = None
         if index_info.input_axis == 0:
-            index_value = parse_index(chunk_input.index_value.to_pandas(),
+            index_value = parse_index(chunk_input.index_value.to_pandas()[:0],
                                       chunk_input, index, store_data=False)
         else:
             dtypes = cls._calc_dtypes(chunk_input.dtypes, index, context)
@@ -567,7 +564,8 @@ class NDArrayFancyIndexHandler(_FancyIndexHandler):
                    index_info: IndexInfo,
                    context: IndexHandlerContext) -> None:
         tileable = context.tileable
-        check_chunks_unknown_shape([tileable], TilesError)
+        if has_unknown_shape(tileable):
+            yield []
 
         # split raw index into chunks on the given axis
         split_info = split_indexes_into_chunks([tileable.nsplits[index_info.input_axis]],
@@ -733,7 +731,8 @@ class LabelNDArrayFancyIndexHandler(_LabelFancyIndexHandler):
                    context: IndexHandlerContext) -> None:
         tileable = context.tileable
         op = context.op
-        check_chunks_unknown_shape([tileable], TilesError)
+        if has_unknown_shape(tileable):
+            yield []
 
         input_axis = index_info.input_axis
         if tileable.ndim == 2:
@@ -877,7 +876,7 @@ class LabelNDArrayFancyIndexHandler(_LabelFancyIndexHandler):
                 shape[1] = len(dtypes)
                 params['shape'] = tuple(shape)
             chunk_op._indexes = indexes
-            chunk_op._stage = OperandStage.agg
+            chunk_op.stage = OperandStage.agg
             out_chunk = chunk_op.new_chunk([concat_chunk], kws=[params])
             if len(out_chunk.shape) != 0:
                 chunk_axis_shapes[out_chunk.index[axis]] = out_chunk.shape[axis]
@@ -916,10 +915,11 @@ class LabelTensorFancyIndexHandler(_LabelFancyIndexHandler):
     def preprocess(self,
                    index_info: IndexInfo,
                    context: IndexHandlerContext) -> None:
-        check_chunks_unknown_shape([index_info.raw_index], TilesError)
+        if has_unknown_shape(index_info.raw_index):
+            yield
         # rechunk index into one
         index_info.unprocessed_raw_index = index_info.raw_index
-        index_info.raw_index = recursive_tile(
+        index_info.raw_index = yield from recursive_tile(
             index_info.raw_index.rechunk(index_info.raw_index.shape))
 
     def process(self,
@@ -989,7 +989,7 @@ class LabelTensorFancyIndexHandler(_LabelFancyIndexHandler):
                     slice_chunks.append(slice_chunk)
 
                 chunk_op = context.op.copy().reset_key()
-                chunk_op._stage = OperandStage.agg
+                chunk_op.stage = OperandStage.agg
                 chunk_op._indexes = (None,) * len(nsplits)
                 chunk_op._fill_value = None
                 assert axis == 0
