@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ import numpy as np
 
 from ... import opcodes as OperandDef
 from ...core import OutputType
-from ...serialize import BoolField, AnyField, StringField
+from ...serialization.serializables import BoolField, AnyField, StringField
 from ..core import IndexValue
 from ..operands import DataFrameOperandMixin, DataFrameOperand, DATAFRAME_TYPE
 from ..utils import parse_index, build_empty_df, build_empty_series, standardize_range_index
@@ -243,7 +243,7 @@ def df_reset_index(df, level=None, drop=False, inplace=False, col_level=0,
         Ensure RangeIndex incremental, when output DataFrame has multiple chunks,
         ensuring index incremental costs more computation,
         so by default, each chunk will have index which starts from 0,
-        setting incremental_index=True，reset_index will garantee that
+        setting incremental_index=True，reset_index will guarantee that
         output DataFrame's index is from 0 to n - 1.
 
     Returns
@@ -402,7 +402,7 @@ def series_reset_index(series, level=None, drop=False, name=None,
         Ensure RangeIndex incremental, when output Series has multiple chunks,
         ensuring index incremental costs more computation,
         so by default, each chunk will have index which starts from 0,
-        setting incremental_index=True，reset_index will garantee that
+        setting incremental_index=True，reset_index will guarantee that
         output Series's index is from 0 to n - 1.
 
     Returns
