@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,8 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import functools
-
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core import OutputType
@@ -25,23 +23,26 @@ class DataFrameMean(DataFrameReductionOperand, DataFrameReductionMixin):
     _func_name = 'mean'
 
     @classmethod
-    def _make_agg_object(cls, op):
-        from .aggregation import mean_function
-        pf = functools.partial(mean_function, skipna=op.skipna)
-        pf.__name__ = cls._func_name
-        return pf
+    def get_reduction_callable(cls, op):
+        skipna = op.skipna
+
+        def mean(x):
+            return x.sum(skipna=skipna) / x.count()
+        return mean
 
 
-def mean_series(df, axis=None, skipna=None, level=None, combine_size=None):
+def mean_series(df, axis=None, skipna=None, level=None, combine_size=None, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameMean(axis=axis, skipna=skipna, level=level, combine_size=combine_size,
-                       output_types=[OutputType.scalar], use_inf_as_na=use_inf_as_na)
+                       output_types=[OutputType.scalar], use_inf_as_na=use_inf_as_na,
+                       method=method)
     return op(df)
 
 
-def mean_dataframe(df, axis=None, skipna=None, level=None, numeric_only=None, combine_size=None):
+def mean_dataframe(df, axis=None, skipna=None, level=None, numeric_only=None,
+                   combine_size=None, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameMean(axis=axis, skipna=skipna, level=level, numeric_only=numeric_only,
                        combine_size=combine_size, output_types=[OutputType.series],
-                       use_inf_as_na=use_inf_as_na)
+                       use_inf_as_na=use_inf_as_na, method=method)
     return op(df)
