@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from distutils.version import LooseVersion
-
 import numpy as np
 import pandas as pd
 
 from .... import opcodes
-from ....serialize import ValueType, AnyField, Int64Field, BoolField, \
-    StringField, Int32Field, KeyField, TupleField, DictField, ListField
-from ....tiles import TilesError
-from ....utils import lazy_import, check_chunks_unknown_shape
+from ....core import recursive_tile
+from ....lib.version import parse as parse_version
+from ....serialization.serializables import FieldTypes, AnyField, Int64Field, \
+    BoolField, StringField, Int32Field, KeyField, TupleField, DictField, ListField
+from ....utils import lazy_import, has_unknown_shape
 from ...operands import DataFrameOperand, DataFrameOperandMixin
 from ...core import DATAFRAME_TYPE
 from ...utils import build_empty_df, build_empty_series, parse_index
@@ -44,8 +43,8 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
     _func_args = TupleField('func_args')
     _func_kwargs = DictField('func_kwargs')
     # for chunks
-    _preds = ListField('preds', ValueType.key)
-    _succs = ListField('succs', ValueType.key)
+    _preds = ListField('preds', FieldTypes.key)
+    _succs = ListField('succs', FieldTypes.key)
 
     def __init__(self, input=None, window=None, min_periods=None, center=None,  # pylint: disable=redefined-builtin
                  win_type=None, on=None, axis=None, closed=None, func=None,
@@ -163,14 +162,14 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
         axis = op.axis
 
         if axis == 0 and inp.ndim == 2:
-            check_chunks_unknown_shape([inp], TilesError)
-            inp = inp.rechunk({1: inp.shape[1]})._inplace_tile()
+            if has_unknown_shape(inp):
+                yield
+            inp = yield from recursive_tile(inp.rechunk({1: inp.shape[1]}))
 
         if is_window_int:
             # if window is integer
             if any(np.isnan(ns) for ns in inp.nsplits[op.axis]):
-                raise TilesError('input DataFrame or Series '
-                                 f'has unknown chunk shape on axis {op.axis}')
+                yield
         else:
             # if window is offset
             # must be aware of index's meta including min and max
@@ -184,8 +183,7 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
                 else:
                     index_value = chunk.columns_value
                 if pd.isnull(index_value.min_val) or pd.isnull(index_value.max_val):
-                    raise TilesError('input DataFrame or Series '
-                                     f'has unknown index meta {op.axis}')
+                    yield
 
         return inp
 
@@ -305,7 +303,7 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
         output_ndim = out.ndim
 
         # check if can be tiled
-        inp = cls._check_can_be_tiled(op, is_window_int)
+        inp = yield from cls._check_can_be_tiled(op, is_window_int)
 
         if inp.ndim == 1 and out.ndim == 1:
             # input series, output series
@@ -411,10 +409,10 @@ class DataFrameRollingAgg(DataFrameOperand, DataFrameOperandMixin):
 
             # fix for pandas 1.2.0
             # see: https://github.com/pandas-dev/pandas/issues/38908
-            # df.rolling().agggregate('skew') modifed original data
+            # df.rolling().aggregate('skew') modified original data
             # so we copy it first for skew only
-            if LooseVersion(pd.__version__) == '1.2.0' and \
-                    op.func in ['skew', 'kurt'] and op.outputs[0].index[0] == 0:
+            if parse_version(pd.__version__) == parse_version('1.2.0') \
+                    and op.func in ['skew', 'kurt'] and op.outputs[0].index[0] == 0:
                 data = data.copy()
 
         r = data.rolling(window=window, min_periods=op.min_periods,
