@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 # register operands
 # import torch first, or some issue emerges,
 # see https://github.com/pytorch/pytorch/issues/2575
-from .contrib import pytorch, tensorflow, xgboost, lightgbm
+from .contrib import pytorch, tensorflow, xgboost, lightgbm, statsmodels
 from .metrics import pairwise
 from . import cluster
 from . import preprocessing
@@ -23,7 +23,7 @@ from . import proxima
 from . import neighbors
 from . import utils
 
-for _mod in [xgboost, tensorflow, pytorch, lightgbm, proxima, neighbors]:
+for _mod in [xgboost, tensorflow, pytorch, lightgbm, proxima, neighbors, statsmodels]:
     _mod.register_op()
 
 del _mod, pairwise, preprocessing, utils
