@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +15,9 @@
 from threading import Thread
 
 from .... import opcodes as OperandDef
-from ....serialize import Int32Field
+from ....core import NotSupportTile
+from ....serialization.serializables import Int32Field
 from ....utils import to_binary
-from ....tiles import NotSupportTile
 from ...operands import LearnOperand, LearnOperandMixin, OutputType
 
 
@@ -47,7 +47,7 @@ class StartTracker(LearnOperand, LearnOperandMixin):
         from .tracker import RabitTracker
 
         env = {'DMLC_NUM_WORKER': op.n_workers}
-        rabit_context = RabitTracker(hostIP=ctx.get_local_address().split(':', 1)[0],
+        rabit_context = RabitTracker(hostIP=ctx.current_address.split(':', 1)[0],
                                      nslave=op.n_workers)
         env.update(rabit_context.slave_envs())
 
