@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,13 +23,13 @@ except ImportError:  # pragma: no cover
 
 from .... import opcodes
 from .... import options
-from ....operands import OperandStage
-from ....serialize import KeyField, BoolField, DictField, Int64Field, AnyField
+from ....core import recursive_tile
+from ....core.operand import OperandStage
+from ....serialization.serializables import KeyField, BoolField, DictField, Int64Field, AnyField
 from ....tensor.core import TensorOrder
 from ....tensor.merge import TensorConcatenate
 from ....tensor.array_utils import as_same_device, device, get_array_module
-from ....tiles import TilesError
-from ....utils import check_chunks_unknown_shape, parse_readable_size
+from ....utils import has_unknown_shape, parse_readable_size, ensure_own_data
 from ...utils import gen_batches
 from ...utils.validation import _num_samples
 from .core import PairwiseDistances
@@ -157,7 +157,9 @@ def _pariwise_distance_chunked(X, Y, reduce_func=None, metric='euclidean',
         else:
             X_chunk = X[sl]
         # call pairwise op's execute method to get the result
-        D_chunk = pairwise_distances(X_chunk, Y, metric=metric, **kwds)
+        D_chunk = pairwise_distances(
+            ensure_own_data(X_chunk), ensure_own_data(Y),
+            metric=metric, **kwds)
         if ((X is Y or Y is None) and metric == 'euclidean'):
             # zeroing diagonal, taking care of aliases of "euclidean",
             # i.e. "l2"
@@ -183,12 +185,10 @@ class PairwiseDistancesTopk(PairwiseDistances):
     _y_offset = Int64Field('y_offset')
 
     def __init__(self, x=None, y=None, k=None, metric=None,
-                 metric_kwargs=None, return_index=None, working_memory=None,
-                 dtype=None, gpu=None, stage=None, **kw):
+                 metric_kwargs=None, return_index=None, working_memory=None, **kw):
         super().__init__(_x=x, _y=y, _k=k, _metric=metric,
                          _metric_kwargs=metric_kwargs, _return_index=return_index,
-                         _working_memory=working_memory,
-                         _dtype=dtype, _gpu=gpu, _stage=stage, **kw)
+                         _working_memory=working_memory, **kw)
 
     @property
     def x(self):
@@ -225,11 +225,11 @@ class PairwiseDistancesTopk(PairwiseDistances):
     @property
     def output_limit(self):
         return 1 if not self._return_index or \
-                    self._stage == OperandStage.map else 2
+                    self.stage == OperandStage.map else 2
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
-        if self._stage != OperandStage.agg:
+        if self.stage != OperandStage.agg:
             self._x = self._inputs[0]
             self._y = self._inputs[1]
         else:
@@ -299,11 +299,12 @@ class PairwiseDistancesTopk(PairwiseDistances):
         k = op.k
 
         if X.chunk_shape[1] > 1:
-            X = X.rechunk({1: X.shape[1]})._inplace_tile()
+            X = yield from recursive_tile(X.rechunk({1: X.shape[1]}))
 
-        check_chunks_unknown_shape([Y], TilesError)
+        if has_unknown_shape(Y):
+            yield
         if Y.chunk_shape[1] > 1:
-            Y = Y.rechunk({1: Y.shape[1]})._inplace_tile()
+            Y = yield from recursive_tile(Y.rechunk({1: Y.shape[1]}))
 
         out_distance_chunks, out_index_chunks = [], []
         y_acc_chunk_shapes = [0] + np.cumsum(Y.nsplits[0]).tolist()
@@ -326,7 +327,7 @@ class PairwiseDistancesTopk(PairwiseDistances):
                     y_chunk = Y.cix[j, 0]
                     chunk_op = op.copy().reset_key()
                     chunk_op._y_offset = y_acc_chunk_shapes[j]
-                    chunk_op._stage = OperandStage.map
+                    chunk_op.stage = OperandStage.map
                     o = chunk_op.new_chunk([x_chunk, y_chunk],
                                            shape=(x_chunk.shape[0], k),
                                            order=TensorOrder.C_ORDER,
@@ -341,7 +342,7 @@ class PairwiseDistancesTopk(PairwiseDistances):
                                              index=(i, 0))
 
                 chunk_op = op.copy().reset_key()
-                chunk_op._stage = OperandStage.agg
+                chunk_op.stage = OperandStage.agg
                 distance_params = {
                     'shape': (x_chunk.shape[0], k),
                     'order': TensorOrder.C_ORDER,
