@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,8 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ..operands import Operand, TileableOperandMixin, Fuse, FuseChunkMixin, \
-    MapReduceOperand, ShuffleProxy, OutputType
+from ..core import OutputType
+from ..core.operand import Operand, TileableOperandMixin, Fuse, FuseChunkMixin, \
+    ShuffleProxy
 from ..tensor.core import TENSOR_TYPE, TENSOR_CHUNK_TYPE
 from ..tensor.operands import TensorOperandMixin
 from ..tensor.fuse import TensorFuseChunk
@@ -76,5 +77,6 @@ class LearnShuffleProxy(ShuffleProxy, LearnOperandMixin):
         if not self.output_types:
             self.output_types = [OutputType.object]
 
-
-LearnMapReduceOperand = MapReduceOperand
+    @classmethod
+    def execute(cls, ctx, op):
+        pass
