@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,10 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-try:
-    from ._data import MinMaxScaler
-    from ._data import minmax_scale
-except ImportError:  # pragma: no cover
-    # sklearn not installed
-    pass
+from ._data import MinMaxScaler
+from ._data import minmax_scale
+from ._label import LabelBinarizer, label_binarize
 from .normalize import normalize
