@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,17 +25,16 @@ import numpy as np
 from .... import opcodes
 from .... import tensor as mt
 from ....config import options
-from ....context import get_context, RunningMode
-from ....core import Base, Entity
+from ....core import ENTITY_TYPE, OutputType, recursive_tile
+from ....core.context import get_context
+from ....core.operand import OperandStage
 from ....lib.filesystem import get_fs, FileSystem
-from ....operands import OutputType, OperandStage
-from ....serialize import KeyField, StringField, Int32Field, Int64Field, \
+from ....serialization.serializables import KeyField, StringField, Int32Field, Int64Field, \
     DictField, AnyField, BytesField, BoolField
 from ....tensor.core import TensorOrder
-from ....tiles import TilesError
-from ....utils import check_chunks_unknown_shape, Timer, ceildiv
+from ....utils import has_unknown_shape, Timer, ceildiv
 from ...operands import LearnOperand, LearnOperandMixin
-from ..core import proxima, validate_tensor
+from ..core import proxima, validate_tensor, get_proxima_type
 
 logger = logging.getLogger(__name__)
 
@@ -69,7 +68,7 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
                          _index_searcher=index_searcher, _index_searcher_params=index_searcher_params,
                          _index_reformer=index_reformer, _index_reformer_params=index_reformer_params,
                          _download_index=download_index, _output_types=output_types, _topk=topk,
-                         _stage=stage, _storage_options=storage_options, **kw)
+                         _storage_options=storage_options, **kw)
         if self._output_types is None:
             self._output_types = [OutputType.tensor, OutputType.tensor]
 
@@ -131,9 +130,9 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
-        if self._stage != OperandStage.agg and not self._download_index:
+        if self.stage != OperandStage.agg and not self._download_index:
             self._tensor = self._inputs[0]
-            if isinstance(self._index, (Base, Entity)):
+            if isinstance(self._index, ENTITY_TYPE):
                 self._index = self._inputs[-1]
 
     def __call__(self, tensor, index):
@@ -161,8 +160,8 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
         download_chunks = defaultdict(list)
         for i, worker in enumerate(workers):
             download_op = op.copy().reset_key()
-            download_op._stage = OperandStage.map
-            download_op._expect_worker = worker
+            download_op.stage = OperandStage.map
+            download_op.expect_worker = worker
             download_op._download_index = True
             download_op._tensor = None
             download_op._index = next(indexes_iter)
@@ -182,7 +181,8 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
         ctx = get_context()
 
         # make sure all inputs have known chunk sizes
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         rechunk_size = dict()
         if tensor.chunk_shape[1] > 1:
@@ -190,7 +190,7 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
         if row_number is not None:
             rechunk_size[0] = tensor.shape[0] // row_number
         if len(rechunk_size) > 0:
-            tensor = tensor.rechunk(rechunk_size)._inplace_tile()
+            tensor = yield from recursive_tile(tensor.rechunk(rechunk_size))
 
         logger.warning(f"query chunks count: {len(tensor.chunks)} ")
 
@@ -208,8 +208,9 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
                 built_indexes.append([next(it) for it in iters])
 
         if hasattr(index, 'op'):
-            index_chunks_workers = [m.workers[0] if m.workers else None for m in
-                                    ctx.get_chunk_metas([c.key for c in index.chunks])]
+            index_chunks_workers = [m['bands'][0][0] for m in
+                                    ctx.get_chunks_meta(
+                                        [c.key for c in index.chunks], fields=['bands'])]
         else:
             index_chunks_workers = [None] * len(built_indexes[0])
 
@@ -219,11 +220,11 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
             for j, chunk_index, worker in \
                     zip(itertools.count(), built_indexes[i], index_chunks_workers):
                 chunk_op = op.copy().reset_key()
-                chunk_op._stage = OperandStage.map
+                chunk_op.stage = OperandStage.map
                 if hasattr(index, 'op'):
-                    chunk_op._expect_worker = worker
+                    chunk_op.expect_worker = worker
                 else:
-                    chunk_op._expect_worker = chunk_index.op.expect_worker
+                    chunk_op.expect_worker = chunk_index.op.expect_worker
                 chunk_op._index = chunk_index
                 chunk_op._tensor = None
                 chunk_kws = [
@@ -259,7 +260,7 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
                     to_combine_distances = tensor_out_chunks[1][k * combine_size: (k + 1) * combine_size]
 
                     chunk_op = op.copy().reset_key()
-                    chunk_op._stage = OperandStage.agg
+                    chunk_op.stage = OperandStage.agg
                     chunk_op._tensor = None
                     chunk_op._index = None
                     agg_chunk_kws = [
@@ -340,20 +341,8 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
             return
 
         inp = ctx[op.tensor.key]
-        check_expect_worker = True
         index_path = ctx[op.inputs[-1].key]
 
-        if hasattr(ctx, 'running_mode') and \
-                ctx.running_mode == RunningMode.distributed and check_expect_worker:
-            # check if the worker to execute is identical to
-            # the worker where built index
-            expect_worker = op.expect_worker
-            curr_worker = ctx.get_local_address()
-            if curr_worker:
-                assert curr_worker == expect_worker, \
-                    f'the worker({curr_worker}) to execute should be identical ' \
-                    f'to the worker({expect_worker}) where built index'
-
         with Timer() as timer:
             flow = proxima.IndexFlow(container_name='MMapFileContainer', container_params={},
                                      searcher_name=op.index_searcher, searcher_params=op.index_searcher_params,
@@ -374,7 +363,9 @@ class ProximaSearcher(LearnOperand, LearnOperandMixin):
             result_pks, result_distances = None, None
             while s_idx < len(vecs):
                 with Timer() as timer_s:
+                    tp = get_proxima_type(vecs.dtype)
                     result_pks_b, result_distances_b = proxima.IndexUtility.ann_search(searcher=flow,
+                                                                                       type=tp,
                                                                                        query=vecs[s_idx:e_idx],
                                                                                        topk=op.topk,
                                                                                        threads=op.threads)
