@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +13,13 @@
 # limitations under the License.
 
 try:
-    from pyarrow.fs import HadoopFileSystem
+    from pyarrow.fs import HadoopFileSystem as _ArrowHadoopFileSystem
     from .arrow import HadoopFileSystem
+    del _ArrowHadoopFileSystem
 except ImportError:  # pragma: no cover
     try:
         # pyarrow < 2.0.0
-        from pyarrow import HadoopFileSystem as HadoopFileSystem
+        from pyarrow import HadoopFileSystem
     except ImportError:
         HadoopFileSystem = None
 
