@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,8 +24,6 @@ from .matrix import SparseMatrix
 from .vector import SparseVector
 from .core import issparse, get_sparse_module
 
-from .coo import COONDArray
-
 
 def asarray(x, shape=None):
     from .core import issparse
@@ -268,6 +266,26 @@ xlogy = partial(_call_bin, 'xlogy')
 
 erf = partial(_call_unary, 'erf')
 
+jv = partial(_call_bin, 'jv')
+jve = partial(_call_bin, 'jve')
+yn = partial(_call_bin, 'yn')
+yv = partial(_call_bin, 'yv')
+yve = partial(_call_bin, 'yve')
+kn = partial(_call_bin, 'kn')
+kv = partial(_call_bin, 'kv')
+kve = partial(_call_bin, 'kve')
+iv = partial(_call_bin, 'iv')
+ive = partial(_call_bin, 'ive')
+hankel1 = partial(_call_bin, 'hankel1')
+hankel1e = partial(_call_bin, 'hankel1e')
+hankel2 = partial(_call_bin, 'hankel2')
+hankel2e = partial(_call_bin, 'hankel2e')
+
+hyp2f1 = partial(call_sparse, 'hyp2f1')
+hyp1f1 = partial(call_sparse, 'hyp1f1')
+hyperu = partial(call_sparse, 'hyperu')
+hyp0f1 = partial(_call_bin, 'hyp0f1')
+
 
 def equal(a, b, **_):
     try:
