@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -219,9 +219,6 @@ SET_IMAG = 197
 
 # special
 SPECIAL = 200
-ERF = 201
-ENTR = 202
-REL_ENTR = 203
 
 # spatial
 PDIST = 231
@@ -389,13 +386,6 @@ DELETE = 740
 
 FUSE = 801
 
-# control
-ENTER = 901
-LEAVE = 902
-FIX_LATEST = 903
-IF_ELSE = 904
-NEXT_ITER = 905
-
 # table like input for tensor
 TABLE_COO = 1003
 # store tensor as coo format
@@ -404,7 +394,6 @@ STORE_COO = 1004
 # shuffle
 SHUFFLE_PROXY = 2001
 DATAFRAME_INDEX_ALIGN = 2004
-UNIQUE_INVERSE_REDUCE = 2013
 
 # indexing
 DATAFRAME_SET_INDEX = 2020
@@ -537,6 +526,13 @@ TYPE_OF_TARGET = 3304
 ACCURACY_SCORE = 3305
 # port detection
 COLLECT_PORTS = 3306
+# unique labels
+UNIQUE_LABELS = 3307
+# PREPROCESSING
+LABEL_BINARIZE = 3308
+# ensemble
+BLOCKWISE_ENSEMBLE_FIT = 3309
+BLOCKWISE_ENSEMBLE_PREDICT = 3310
 
 # Remote Functions and class
 REMOTE_FUNCATION = 5001
