@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from mars.core.base import MarsError
+from ..core.base import MarsError
 
 
 class ActorPoolNotStarted(MarsError):
