@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -89,9 +89,6 @@ class TensorTreeAdd(TensorMultiOp):
     _op_type_ = OperandDef.TREE_ADD
     _func_name = 'add'
 
-    def __init__(self, sparse=False, **kw):
-        super().__init__(_sparse=sparse, **kw)
-
     @classmethod
     def _is_sparse(cls, *args):
         if args and all(hasattr(x, 'issparse') and x.issparse() for x in args):
