@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,8 @@ import itertools
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import KeyField, StringField
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...serialization.serializables import KeyField, StringField
+from ...utils import has_unknown_shape
 from ..utils import unify_chunks, broadcast_shape
 from ..operands import TensorOperand, TensorOperandMixin
 from ..datasource import tensor as astensor
@@ -37,8 +36,8 @@ class TensorCopyTo(TensorOperand, TensorOperandMixin):
     _casting = StringField('casting')
     _where = KeyField('where')
 
-    def __init__(self, casting=None, dtype=None, gpu=None, sparse=None, **kw):
-        super().__init__(_casting=casting, _dtype=dtype, _gpu=gpu, _sparse=sparse, **kw)
+    def __init__(self, casting=None, **kw):
+        super().__init__(_casting=casting, **kw)
 
     @property
     def src(self):
@@ -89,9 +88,9 @@ class TensorCopyTo(TensorOperand, TensorOperandMixin):
         if not isinstance(dst, Tensor):
             raise TypeError('dst has to be a Tensor')
 
-        self._dtype = dst.dtype
-        self._gpu = dst.op.gpu
-        self._sparse = dst.issparse()
+        self.dtype = dst.dtype
+        self.gpu = dst.op.gpu
+        self.sparse = dst.issparse()
 
         if not np.can_cast(src.dtype, dst.dtype, casting=self.casting):
             raise TypeError(f'Cannot cast array from {src.dtype!r} to {dst.dtype!r} '
@@ -117,8 +116,10 @@ class TensorCopyTo(TensorOperand, TensorOperandMixin):
 
     @classmethod
     def tile(cls, op):
-        check_chunks_unknown_shape(op.inputs, TilesError)
-        inputs = unify_chunks(*[(input, list(range(input.ndim))[::-1]) for input in op.inputs])
+        if has_unknown_shape(*op.inputs):
+            yield
+        inputs = yield from unify_chunks(
+            *[(input, list(range(input.ndim))[::-1]) for input in op.inputs])
         output = op.outputs[0]
 
         chunk_shapes = [t.chunk_shape if hasattr(t, 'chunk_shape') else t
