@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,11 +18,11 @@ from functools import partial
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...tiles import TilesError
-from ...serialize import ValueType, Int32Field, \
+from ...core import recursive_tile
+from ...core.operand import OperandStage
+from ...serialization.serializables import FieldTypes, Int32Field, \
     ListField, StringField, BoolField, AnyField
-from ...operands import OperandStage
-from ...utils import get_shuffle_input_keys_idxes, flatten, stack_back
+from ...utils import flatten, stack_back
 from ..core import TensorOrder
 from ..operands import TensorOperand, TensorMapReduceOperand, \
     TensorShuffleProxy, TensorOperandMixin
@@ -47,13 +47,13 @@ class PSRSOperandMixin:
         if not has_unknown_shape:
             axis_chunk_shape = min(axis_chunk_shape, int(np.sqrt(axis_shape)))
             if np.isnan(axis_shape) or any(np.isnan(s) for s in in_data.nsplits[op.axis]):
-                raise TilesError('fail to tile because either the shape of '
-                                 f'input data on axis {op.axis} has unknown shape or chunk shape')
+                yield
             chunk_size = int(axis_shape / axis_chunk_shape)
             chunk_sizes = [chunk_size for _ in range(int(axis_shape // chunk_size))]
             if axis_shape % chunk_size > 0:
                 chunk_sizes[-1] += axis_shape % chunk_size
-            in_data = in_data.rechunk({op.axis: tuple(chunk_sizes)})._inplace_tile()
+            in_data = yield from recursive_tile(
+                in_data.rechunk({op.axis: tuple(chunk_sizes)}))
             axis_chunk_shape = in_data.chunk_shape[op.axis]
 
         left_chunk_shape = in_data.chunk_shape[:op.axis] + in_data.chunk_shape[op.axis + 1:]
@@ -209,7 +209,7 @@ class TensorPSRSOperandMixin(TensorOperandMixin, PSRSOperandMixin):
                                                    stage=OperandStage.reduce,
                                                    axis=op.axis, order=op.order,
                                                    kind=kind,
-                                                   shuffle_key=str(i),
+                                                   reducer_index=(i,),
                                                    dtype=partition_chunk.dtype,
                                                    gpu=partition_chunk.op.gpu,
                                                    need_align=need_align)
@@ -283,7 +283,7 @@ class TensorPSRSOperandMixin(TensorOperandMixin, PSRSOperandMixin):
         for i, align_map_chunk in enumerate(align_map_chunks):
             align_reduce_op = PSRSAlign(return_value=return_value, return_indices=return_indices,
                                         stage=OperandStage.reduce, axis=op.axis,
-                                        shuffle_key=str(i), dtype=align_map_chunk.dtype,
+                                        reducer_index=(i,), dtype=align_map_chunk.dtype,
                                         gpu=align_map_chunk.op.gpu)
             idx = list(out_idx)
             idx.insert(op.axis, i)
@@ -347,7 +347,7 @@ class PSRSSortRegularSample(TensorOperand, TensorOperandMixin):
     _op_type_ = OperandDef.PSRS_SORT_REGULAR_SMAPLE
 
     _axis = Int32Field('axis')
-    _order = ListField('order', ValueType.string)
+    _order = ListField('order', FieldTypes.string)
     _kind = StringField('kind')
     _return_indices = BoolField('return_indices')
     _n_partition = Int32Field('n_partition')
@@ -437,7 +437,7 @@ class PSRSConcatPivot(TensorOperand, TensorOperandMixin):
     _op_type_ = OperandDef.PSRS_CONCAT_PIVOT
 
     _axis = Int32Field('axis')
-    _order = ListField('order', ValueType.string)
+    _order = ListField('order', FieldTypes.string)
     _kind = StringField('kind')
 
     def __init__(self, axis=None, order=None, kind=None, dtype=None, gpu=None, **kw):
@@ -489,7 +489,7 @@ class PSRSShuffle(TensorMapReduceOperand, TensorOperandMixin):
 
     # for shuffle map
     _axis = Int32Field('axis')
-    _order = ListField('order', ValueType.string)
+    _order = ListField('order', FieldTypes.string)
     _n_partition = Int32Field('n_partition')
     _input_sorted = BoolField('input_sorted')
 
@@ -499,12 +499,11 @@ class PSRSShuffle(TensorMapReduceOperand, TensorOperandMixin):
 
     def __init__(self, return_value=None, return_indices=None,
                  axis=None, order=None, n_partition=None, input_sorted=None,
-                 kind=None, need_align=None, stage=None, shuffle_key=None,
-                 dtype=None, gpu=None, **kw):
+                 kind=None, need_align=None, **kw):
         super().__init__(_return_value=return_value, _return_indices=return_indices,
                          _axis=axis, _order=order, _n_partition=n_partition,
-                         _input_sorted=input_sorted, _kind=kind, _need_align=need_align,
-                         _stage=stage, _shuffle_key=shuffle_key, _dtype=dtype, _gpu=gpu, **kw)
+                         _input_sorted=input_sorted, _kind=kind,
+                         _need_align=need_align, **kw)
 
     @property
     def return_value(self):
@@ -540,7 +539,7 @@ class PSRSShuffle(TensorMapReduceOperand, TensorOperandMixin):
 
     @property
     def output_limit(self):
-        if self._stage == OperandStage.map:
+        if self.stage == OperandStage.map:
             return 1
         else:
             limit = int(bool(self._return_value)) + \
@@ -602,12 +601,11 @@ class PSRSShuffle(TensorMapReduceOperand, TensorOperandMixin):
                             reduce_out.append(indices)
                         reduce_outputs[i][idx] = tuple(reduce_out)
             for i in range(op.n_partition):
-                ctx[(out.key, str(i))] = tuple(reduce_outputs[i].ravel())
+                ctx[out.key, (i,)] = tuple(reduce_outputs[i].ravel())
 
     @classmethod
-    def _execute_reduce(cls, ctx, op):
-        input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-        raw_inputs = [ctx[(input_key, op.shuffle_key)] for input_key in input_keys]
+    def _execute_reduce(cls, ctx, op: "PSRSShuffle"):
+        raw_inputs = list(op.iter_mapper_data(ctx))
         # flatten inputs
         flatten_inputs = flatten(raw_inputs)
         inputs, device_id, xp = as_same_device(flatten_inputs, device=op.device, ret_extra=True)
@@ -691,14 +689,12 @@ class PSRSAlign(TensorMapReduceOperand, TensorOperandMixin):
     _return_value = BoolField('return_value')
     _return_indices = BoolField('return_indices')
     _axis = Int32Field('axis')
-    _output_sizes = ListField('output_sizes', ValueType.int32)
+    _output_sizes = ListField('output_sizes', FieldTypes.int32)
 
     def __init__(self, return_value=None, return_indices=None, axis=None,
-                 output_sizes=None, stage=None, shuffle_key=None,
-                 dtype=None, gpu=None, **kw):
+                 output_sizes=None, **kw):
         super().__init__(_return_value=return_value, _return_indices=return_indices,
-                         _axis=axis, _output_sizes=output_sizes, _stage=stage,
-                         _shuffle_key=shuffle_key, _dtype=dtype, _gpu=gpu, **kw)
+                         _axis=axis, _output_sizes=output_sizes, **kw)
 
     @property
     def return_value(self):
@@ -718,7 +714,7 @@ class PSRSAlign(TensorMapReduceOperand, TensorOperandMixin):
 
     @property
     def output_limit(self):
-        if self._stage == OperandStage.map:
+        if self.stage == OperandStage.map:
             return 1
         else:
             return int(bool(self._return_value)) + int(bool(self._return_indices))
@@ -778,14 +774,12 @@ class PSRSAlign(TensorMapReduceOperand, TensorOperandMixin):
                         ret.append(tuple(item))
                     else:
                         ret.append(ar)
-                ctx[(op.outputs[0].key, str(idx))] = tuple(ret)
+                ctx[op.outputs[0].key, (idx,)] = tuple(ret)
 
     @classmethod
-    def _execute_reduce(cls, ctx, op):
-        in_chunk = op.inputs[0]
+    def _execute_reduce(cls, ctx, op: "PSRSAlign"):
         axis = op.axis
-        input_keys, _ = get_shuffle_input_keys_idxes(in_chunk)
-        raw_inputs = [ctx[(input_key, op.shuffle_key)] for input_key in input_keys]
+        raw_inputs = list(op.iter_mapper_data(ctx))
         flatten_inputs = flatten(raw_inputs)
         inputs, device_id, xp = as_same_device(flatten_inputs, device=op.device, ret_extra=True)
         inputs = stack_back(flatten_inputs, raw_inputs)
