@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,11 +18,11 @@ import numpy as np
 
 from ... import opcodes as OperandDef
 from ...config import options
-from ...core import ExecutableTuple
-from ...serialize import ValueType, KeyField, Int64Field, Int32Field, \
-    BoolField, StringField, ListField
-from ...operands import OperandStage
-from ...utils import ceildiv, flatten, recursive_tile
+from ...core import ExecutableTuple, recursive_tile
+from ...core.operand import OperandStage
+from ...serialization.serializables import FieldTypes, KeyField, \
+    Int64Field, Int32Field, BoolField, StringField, ListField
+from ...utils import ceildiv, flatten
 from ..array_utils import as_same_device, device
 from ..core import TensorOrder
 from ..datasource import tensor as astensor
@@ -39,9 +39,9 @@ class TensorTopk(TensorOperand, TensorOperandMixin):
     _axis = Int32Field('axis')
     _largest = BoolField('largest')
     _sorted = BoolField('sorted')
-    _order = ListField('order', ValueType.string)
+    _order = ListField('order', FieldTypes.string)
     _parallel_kind = StringField('parallel_kind')
-    _psrs_kinds = ListField('psrs_kinds', ValueType.string)
+    _psrs_kinds = ListField('psrs_kinds', FieldTypes.string)
     _return_value = BoolField('return_value')
     _return_indices = BoolField('return_indices')
     _axis_offset = Int64Field('axis_offset',
@@ -50,12 +50,11 @@ class TensorTopk(TensorOperand, TensorOperandMixin):
 
     def __init__(self, k=None, axis=None, largest=None, sorted=None, order=None,
                  parallel_kind=None, psrs_kinds=None, return_value=None, return_indices=None,
-                 axis_offset=None, stage=None, dtype=None, gpu=None, **kw):
+                 axis_offset=None, **kw):
         super().__init__(_k=k, _axis=axis, _largest=largest, _sorted=sorted,
                          _parallel_kind=parallel_kind, _psrs_kinds=psrs_kinds,
                          _return_value=return_value, _return_indices=return_indices,
-                         _order=order, _axis_offset=axis_offset, _stage=stage,
-                         _dtype=dtype, _gpu=gpu, **kw)
+                         _order=order, _axis_offset=axis_offset, **kw)
 
     @property
     def input(self):
@@ -103,7 +102,7 @@ class TensorTopk(TensorOperand, TensorOperandMixin):
 
     @property
     def output_limit(self):
-        if self._stage != OperandStage.agg:
+        if self.stage != OperandStage.agg:
             return 1
         else:
             return int(bool(self._return_value)) + int(bool(self._return_indices))
@@ -192,7 +191,7 @@ class TensorTopk(TensorOperand, TensorOperandMixin):
         else:
             ret = [r[base_slcs + (slice(op.k),)] for r in ret]
 
-        ret = [recursive_tile(r) for r in ret]
+        ret = yield from recursive_tile(ret)
         new_op = op.copy()
         kws = [o.params for o in op.outputs]
         if return_value:
@@ -218,14 +217,14 @@ class TensorTopk(TensorOperand, TensorOperandMixin):
             # whenever return_indices, value is required
             chunk_op._return_value = True
             if axis_offset is not None:
-                chunk_op._stage = OperandStage.map
+                chunk_op.stage = OperandStage.map
             else:
-                chunk_op._stage = OperandStage.combine
+                chunk_op.stage = OperandStage.combine
             return chunk_op.new_chunk([input_chunk], shape=tuple(shape),
                                       order=input_chunk.order,
                                       index=chunk_index)
         else:
-            chunk_op._stage = OperandStage.agg
+            chunk_op.stage = OperandStage.agg
             kws = []
             if op.return_value:
                 kws.append({
@@ -342,7 +341,7 @@ class TensorTopk(TensorOperand, TensorOperandMixin):
         else:
             assert parallel_kind == 'psrs'
             op._parallel_kind = 'psrs'
-            return cls._tile_via_psrs(op)
+            return (yield from cls._tile_via_psrs(op))
 
     @classmethod
     def execute(cls, ctx, op):
