@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,8 @@ import itertools
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import KeyField
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...serialization.serializables import KeyField
+from ...utils import has_unknown_shape
 from ..utils import broadcast_shape, unify_chunks
 from ..array_utils import as_same_device, device
 from ..core import TENSOR_TYPE
@@ -37,9 +36,6 @@ class TensorWhere(TensorOperand, TensorOperandMixin):
     _x = KeyField('x')
     _y = KeyField('y')
 
-    def __init__(self, dtype=None, **kw):
-        super().__init__(_dtype=dtype, **kw)
-
     @property
     def condition(self):
         return self._condition
@@ -64,8 +60,10 @@ class TensorWhere(TensorOperand, TensorOperandMixin):
 
     @classmethod
     def tile(cls, op):
-        check_chunks_unknown_shape(op.inputs, TilesError)
-        inputs = unify_chunks(*[(input, list(range(input.ndim))[::-1]) for input in op.inputs])
+        if has_unknown_shape(*op.inputs):
+            yield
+        inputs = yield from unify_chunks(
+            *[(input, list(range(input.ndim))[::-1]) for input in op.inputs])
         chunk_shapes = [t.chunk_shape if isinstance(t, TENSOR_TYPE) else t
                         for t in inputs]
         out_chunk_shape = broadcast_shape(*chunk_shapes)
@@ -178,5 +176,5 @@ def where(condition, x=None, y=None):
         return broadcast_to(x if condition else y, shape).astype(dtype)
     else:
         condition = astensor(condition)
-        op = TensorWhere(dtype)
+        op = TensorWhere(dtype=dtype)
         return op(condition, x, y, shape=shape)
