@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,17 +13,18 @@
 # limitations under the License.
 
 import numpy as np
+
+from ... import opcodes as OperandDef
+from ...serialization.serializables import AnyField
+from ...config import options
+from ...lib.filesystem import open_file, glob, file_size
+from ...utils import ceildiv, ModulePlaceholder
+from ..operands import TensorOperandMixin, TensorOperand
+
 try:
     from PIL import Image
 except ImportError:
-    Image = None
-
-from ... import opcodes as OperandDef
-from ...serialize import AnyField
-from ...config import options
-from ...lib.filesystem import open_file, glob, file_size
-from ...utils import ceildiv
-from ..operands import TensorOperandMixin, TensorOperand
+    Image = ModulePlaceholder('PIL')
 
 
 def _read_image(fpath):
@@ -35,8 +36,8 @@ class TensorImread(TensorOperand, TensorOperandMixin):
 
     _filepath = AnyField('filepath')
 
-    def __init__(self, filepath=None, dtype=None, **kwargs):
-        super().__init__(_filepath=filepath, _dtype=dtype, **kwargs)
+    def __init__(self, filepath=None, **kwargs):
+        super().__init__(_filepath=filepath, **kwargs)
 
     @property
     def filepath(self):
