@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,9 +18,9 @@ import numpy as np
 from numpy.linalg import LinAlgError
 
 from ... import opcodes as OperandDef
-from ...serialize import BoolField, KeyField
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...core import recursive_tile
+from ...serialization.serializables import BoolField, KeyField
+from ...utils import has_unknown_shape
 from ..array_utils import device, as_same_device, cp
 from ..operands import TensorOperand, TensorOperandMixin
 from ..datasource import tensor as astensor
@@ -36,8 +36,8 @@ class TensorSolveTriangular(TensorOperand, TensorOperandMixin):
     _lower = BoolField('lower')
     _strict = BoolField('strict')
 
-    def __init__(self, lower=None, dtype=None, sparse=False, strict=None, **kw):
-        super().__init__(_lower=lower, _dtype=dtype, _sparse=sparse, _strict=strict, **kw)
+    def __init__(self, lower=None, strict=None, **kw):
+        super().__init__(_lower=lower, _strict=strict, **kw)
 
     @property
     def a(self):
@@ -69,12 +69,13 @@ class TensorSolveTriangular(TensorOperand, TensorOperandMixin):
         from ..arithmetic.utils import chunk_tree_add
         from .dot import TensorDot
 
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         a, b = op.a, op.b
         unified_nsplit = decide_unify_split(a.nsplits[0], a.nsplits[1], b.nsplits[0])
-        a = a.rechunk((unified_nsplit, unified_nsplit))._inplace_tile()
-        b = b.rechunk((unified_nsplit,) + b.nsplits[1:])._inplace_tile()
+        a = yield from recursive_tile(a.rechunk((unified_nsplit, unified_nsplit)))
+        b = yield from recursive_tile(b.rechunk((unified_nsplit,) + b.nsplits[1:]))
 
         b_multi_dim = b.ndim > 1
         b_hsplits = b.chunk_shape[1] if b_multi_dim else 1
