@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,7 +20,8 @@ from collections.abc import Iterable
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import AnyField, BoolField, StringField, TupleField, SliceField
+from ...serialization.serializables import AnyField, BoolField, \
+    StringField, TupleField, SliceField
 from ..array_utils import device, as_same_device
 from ..utils import validate_axis, unify_chunks
 from ..datasource import tensor as astensor
@@ -62,14 +63,13 @@ class TensorConcatenate(TensorOperand, TensorOperandMixin):
     _partition_slice = SliceField('partition_slice')
     _total_shape = TupleField('total_shape')
 
-    def __init__(self, axis=None, dtype=None, mmap=None, file_prefix=None, create_mmap_file=None,
-                 partition_slice=None, total_shape=None, sparse=False, gpu=None, **kw):
-        super().__init__(_axis=axis, _dtype=dtype, _mmap=mmap,
+    def __init__(self, axis=None, mmap=None, file_prefix=None, create_mmap_file=None,
+                 partition_slice=None, total_shape=None, **kw):
+        super().__init__(_axis=axis, _mmap=mmap,
                          _file_prefix=file_prefix,
                          _create_mmap_file=create_mmap_file,
                          _partition_slice=partition_slice,
-                         _total_shape=total_shape,
-                         _gpu=gpu, _sparse=sparse, **kw)
+                         _total_shape=total_shape, **kw)
 
     @property
     def axis(self):
@@ -124,7 +124,7 @@ class TensorConcatenate(TensorOperand, TensorOperandMixin):
         c = itertools.count(inputs[0].ndim)
         tensor_axes = [(t, tuple(i if i != axis else next(c) for i in range(t.ndim)))
                        for t in inputs]
-        inputs = unify_chunks(*tensor_axes)
+        inputs = yield from unify_chunks(*tensor_axes)
 
         out_chunk_shape = [0 if i == axis else inputs[0].chunk_shape[i]
                            for i in range(inputs[0].ndim)]
