@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,8 @@ import itertools
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import Int32Field
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape
+from ...serialization.serializables import Int32Field
+from ...utils import has_unknown_shape
 from ..utils import unify_chunks, check_out_param
 from ..array_utils import as_same_device, device
 from ..operands import TensorOperand, TensorOperandMixin
@@ -34,8 +33,8 @@ class TensorStack(TensorOperand, TensorOperandMixin):
 
     _axis = Int32Field('axis')
 
-    def __init__(self, axis=None, dtype=None, sparse=False, **kw):
-        super().__init__(_axis=axis, _dtype=dtype, _sparse=sparse, **kw)
+    def __init__(self, axis=None, **kw):
+        super().__init__(_axis=axis, **kw)
 
     @property
     def axis(self):
@@ -62,13 +61,14 @@ class TensorStack(TensorOperand, TensorOperandMixin):
     def tile(cls, op):
         from ..indexing.slice import TensorSlice
 
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         if len(set([inp.shape for inp in op.inputs])) != 1:
             # check shape again when input has unknown shape
             raise ValueError('all input tensors must have the same shape')
 
-        inputs = unify_chunks(*op.inputs)
+        inputs = yield from unify_chunks(*op.inputs)
         output = op.outputs[0]
         axis = op.axis
 
