@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,9 +12,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ..serialize import DataTypeField
-from ..operands import Operand, TileableOperandMixin, HasInput, ShuffleProxy, MapReduceOperand, \
-    Fuse, OutputType
+from ..serialization.serializables import DataTypeField
+from ..core import OutputType
+from ..core.operand import Operand, TileableOperandMixin, HasInput, \
+    ShuffleProxy, MapReduceOperand, Fuse
 from ..utils import calc_nsplits
 
 
@@ -63,48 +64,33 @@ class TensorOperandMixin(TileableOperandMixin):
 
 class TensorOperand(Operand):
     _output_type_ = OutputType.tensor
-    _dtype = DataTypeField('dtype')
 
-    @property
-    def dtype(self):
-        return getattr(self, '_dtype', None)
+    dtype = DataTypeField('dtype', default=None)
 
 
 class TensorHasInput(HasInput):
     _output_type_ = OutputType.tensor
-    _dtype = DataTypeField('dtype')
 
-    @property
-    def dtype(self):
-        return getattr(self, '_dtype', None)
+    dtype = DataTypeField('dtype', default=None)
 
 
 class TensorShuffleProxy(ShuffleProxy, TensorOperandMixin):
     _output_type_ = OutputType.tensor
-    _dtype = DataTypeField('dtype')
 
-    def __init__(self, dtype=None, **kwargs):
-        kwargs['_dtype'] = kwargs.get('_dtype', dtype)
-        super().__init__(**kwargs)
+    dtype = DataTypeField('dtype', default=None)
 
-    @property
-    def dtype(self):
-        return getattr(self, '_dtype', None)
+    @classmethod
+    def execute(cls, ctx, op):
+        pass
 
 
 class TensorMapReduceOperand(MapReduceOperand):
     _output_type_ = OutputType.tensor
-    _dtype = DataTypeField('dtype')
 
-    @property
-    def dtype(self):
-        return getattr(self, '_dtype', None)
+    dtype = DataTypeField('dtype', default=None)
 
 
 class TensorFuse(Fuse):
     _output_type_ = OutputType.tensor
-    _dtype = DataTypeField('dtype')
 
-    @property
-    def dtype(self):
-        return getattr(self, '_dtype', None)
+    dtype = DataTypeField('dtype', default=None)
