@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,10 +20,10 @@ import logging
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...operands import OperandStage
-from ...serialize import KeyField, TupleField, StringField, ValueType
-from ...tiles import TilesError
-from ...utils import get_shuffle_input_keys_idxes, check_chunks_unknown_shape, recursive_tile
+from ...core import recursive_tile
+from ...core.operand import OperandStage
+from ...serialization.serializables import FieldTypes, KeyField, TupleField, StringField
+from ...utils import has_unknown_shape
 from ..array_utils import as_same_device, device
 from ..datasource import tensor as astensor
 from ..operands import TensorOperandMixin, TensorMapReduceOperand, TensorShuffleProxy
@@ -36,20 +36,17 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
     _op_type_ = OperandDef.RESHAPE
 
     _input = KeyField('input')
-    _newshape = TupleField('newshape', ValueType.int64)
+    _newshape = TupleField('newshape', FieldTypes.int64)
     _order = StringField('order')
 
-    _axis_offsets = TupleField('axis_offsets', ValueType.uint64)
-    _oldshape = TupleField('oldshape', ValueType.uint64)
-    _new_chunk_size = TupleField('new_chunk_size', ValueType.uint64)
+    _axis_offsets = TupleField('axis_offsets', FieldTypes.uint64)
+    _oldshape = TupleField('oldshape', FieldTypes.uint64)
+    _new_chunk_size = TupleField('new_chunk_size', FieldTypes.uint64)
 
     def __init__(self, newshape=None, order=None, axis_offsets=None, oldshape=None,
-                 new_chunk_size=None, stage=None, shuffle_key=None, dtype=None,
-                 create_view=None, **kw):
+                 new_chunk_size=None, **kw):
         super().__init__(_newshape=newshape, _order=order, _axis_offsets=axis_offsets,
-                         _oldshape=oldshape, _new_chunk_size=new_chunk_size,
-                         _stage=stage, _shuffle_key=shuffle_key, _dtype=dtype,
-                         _create_view=create_view, **kw)
+                         _oldshape=oldshape, _new_chunk_size=new_chunk_size, **kw)
 
     @property
     def input(self):
@@ -218,9 +215,7 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
 
         for chunk_shape, chunk_idx in zip(itertools.product(*out_nsplits),
                                           itertools.product(*chunk_size_idxes)):
-            shuffle_key = ','.join(str(o) for o in chunk_idx)
-            chunk_op = TensorReshape(stage=OperandStage.reduce, dtype=tensor.dtype,
-                                     shuffle_key=shuffle_key)
+            chunk_op = TensorReshape(stage=OperandStage.reduce, dtype=tensor.dtype)
             shuffle_outputs.append(chunk_op.new_chunk([proxy_chunk], shape=chunk_shape,
                                                       order=tensor.order, index=chunk_idx))
 
@@ -234,7 +229,8 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
         tensor = op.outputs[0]
 
         # check unknown shape
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         if any(np.isnan(s) for s in tensor.shape):
             # -1 exists in newshape and input tensor has unknown shape
@@ -249,7 +245,7 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
             if getattr(op, '_reshape_with_shuffle', True):
                 result.op.extra_params['_reshape_with_shuffle'] = True
             result = result.transpose()
-            return [recursive_tile(result)]
+            return [(yield from recursive_tile(result))]
 
         if len(in_tensor.chunks) == 1:
             # 1 chunk
@@ -263,7 +259,8 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
         try:
             rechunk_nsplits, reshape_nsplits = cls._gen_reshape_rechunk_nsplits(
                 in_tensor.shape, tensor.shape, in_tensor.nsplits)
-            rechunked_tensor = in_tensor.rechunk(rechunk_nsplits)._inplace_tile()
+            rechunked_tensor = yield from recursive_tile(
+                in_tensor.rechunk(rechunk_nsplits))
             in_idxes = itertools.product(*[range(len(s)) for s in rechunk_nsplits])
             out_idxes = itertools.product(*[range(len(s)) for s in reshape_nsplits])
             out_shape = itertools.product(*[s for s in reshape_nsplits])
@@ -285,8 +282,9 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
                 return cls._tile_as_shuffle(op)
 
             # shape incompatible, we will first do flatten, then reshape to the new shape
-            return [in_tensor.reshape(-1, order=tensor.op.order)._inplace_tile().reshape(
-                tensor.shape, order=tensor.op.order)._inplace_tile()]
+            return [(yield from recursive_tile(
+                in_tensor.reshape(-1, order=tensor.op.order)
+                    .reshape(tensor.shape, order=tensor.op.order)))]
 
     @classmethod
     def estimate_size(cls, ctx, op):
@@ -300,7 +298,7 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
         elif op.stage == OperandStage.reduce:
             sum_size = 0
             for shuffle_input in chunk.inputs[0].inputs or ():
-                key = (shuffle_input.key, chunk.op.shuffle_key)
+                key = (shuffle_input.key, chunk.index)
                 if ctx.get(key) is not None:
                     sum_size += ctx[key][0]
                 else:
@@ -376,30 +374,19 @@ class TensorReshape(TensorMapReduceOperand, TensorOperandMixin):
             for idx, dim_chunk_count in enumerate(reversed(dim_chunk_counts)):
                 t, target_chunk_idx[idx] = divmod(t, dim_chunk_count)
             target_chunk_idx.reverse()
-            group_key = ','.join(str(v) for v in target_chunk_idx)
 
-            ctx[(chunk.key, group_key)] = group_indices + (group_data,)
+            ctx[chunk.key, tuple(target_chunk_idx)] = group_indices + (group_data,)
 
     @classmethod
-    def _execute_reduce(cls, ctx, op):
+    def _execute_reduce(cls, ctx, op: "TensorReshape"):
         chunk = op.outputs[0]
         try:
             result_array = ctx[chunk.key]
         except KeyError:
             result_array = np.zeros(chunk.shape, dtype=chunk.dtype,
                                     order=chunk.order.value)
-
-        in_chunk = chunk.inputs[0]
-        input_keys, _ = get_shuffle_input_keys_idxes(in_chunk)
-
-        shuffle_key = chunk.op.shuffle_key
-        for input_key in input_keys:
-            key = (input_key, shuffle_key)
-            if ctx.get(key) is not None:
-                data_tuple = ctx[key]
-                result_array[data_tuple[:-1]] = data_tuple[-1]
-            else:
-                ctx[key] = None
+        for data_tuple in op.iter_mapper_data(ctx, skip_none=True):
+            result_array[data_tuple[:-1]] = data_tuple[-1]
         ctx[chunk.key] = result_array
 
     @classmethod
