@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,10 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import scipy.special as spspecial
+
 from ..utils import infer_dtype, implement_scipy
-from .core import spspecial, TensorSpecialBinOp
+from .core import TensorSpecialBinOp, _register_special_op
 
 
+@_register_special_op
 class TensorXLogY(TensorSpecialBinOp):
     _func_name = 'xlogy'
 
